"""
create_time: 2022-09
author: Jacky Lee
description: 并查集
"""


class UnionFind:
    def __init__(self, n: int):
        self.uf = list(range(n + 1))
        self.rank = [1] * (n + 1)  # 规模

    def find(self, x: int) -> int:
        r = x
        if self.uf[x] != x:
            x = self.uf[x]
        # 路径压缩
        while r != x:
            self.uf[r], r = x, self.uf[r]
        return x

    def union(self, x: int, y: int) -> None:
        fx, fy = self.find(x), self.find(y)
        if fx == fy:
            return
        # 小规模往大规模合并
        if self.rank[fx] < self.rank[fy]:
            fx, fy = fy, fx
        self.rank[fx] += self.rank[fy]
        self.uf[fy] = fx

    def is_connected(self, x: int, y: int) -> bool:
        return self.find(x) == self.find(y)



import logging
import tempfile
from pathlib import Path
from typing import Callable

import inquirer

from pydevman.query.cache import JsonCache

DEFAULT_TMP_CACHE = Path(tempfile.gettempdir(), "pydevman.global.cache.json")

log = logging.getLogger(__name__)


class QueryCache:
    def __init__(self, path: Path = None):
        self.path = path or DEFAULT_TMP_CACHE
        self.cache = JsonCache(self.path)
        log.debug(f"创建缓存 {self.path}")

    def clear_cache(self): ...

    def query_list(self, func: str, name: str, msg: str, validate: Callable = None):
        choices = self.cache.arg_get_list(func, name)
        if choices:
            res = inquirer.list_input(msg, choices=choices, other=True)
        else:
            res = inquirer.text(msg)
        # if validate and validate(res):
        if validate is None or validate(res):
            self.cache.arg_upsert(func, name, res)
        return res

    def query_check(self, func: str, name: str, msg: str):
        choices = self.cache.arg_get_list(func, name)
        res = inquirer.checkbox(msg, choices=choices, other=True)
        self.cache.arg_upsert_list(func, name, res)
        return res


def confirm() -> bool:
    while True:
        flag = input("是否继续(y/n)...").strip().lower()  # 获取用户输入并处理为小写
        if flag.startswith("y"):  # 如果以'y'开头
            log.info("继续执行...")
            return True
        elif flag.startswith("n"):  # 如果以'n'开头
            log.info("退出程序...")
            return False
        log.info("无效输入，请输入'y'或'n'。")  # 提示用户输入无效，继续询问



import logging
from pathlib import Path

from send2trash import send2trash

from pydevman.file.common import assert_path_exist_and_is_dir, is_empty_directory

log = logging.getLogger(__name__)


def del_empty_dir(path: Path, dry):
    if not is_empty_directory(path):
        return
    log.debug(f"删除空目录({not dry}): 目标文件夹='{path}'")
    if not dry:
        send2trash(path)


def del_empty_dir_recursive(dst: Path, dry: bool):
    assert_path_exist_and_is_dir(dst)
    log.debug(f"遍历目录: 目标文件夹='{dst}'")
    for path in dst.iterdir():
        del_empty_dir(path, dry)


def del_dir(dst: Path, dry: bool):
    # 校验
    assert_path_exist_and_is_dir(dst)
    # 记录
    log.debug(f"删除到回收站({not dry}): 目标文件夹='{dst}'")
    # 操作
    if not dry:
        send2trash(dst)



from datetime import datetime

from sqlalchemy import DateTime, Integer, String
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column
from sqlalchemy.sql import func


class Base(DeclarativeBase): ...


class IdMixin:
    id: Mapped[int] = mapped_column(
        Integer, primary_key=True, autoincrement=True, comment="主键"
    )


class TimeMixin:
    """表结构基础:包含表基础字段(创建时间、更新时间和软删除标识)"""

    create_time: Mapped[datetime] = mapped_column(
        DateTime, server_default=func.now(), comment="创建时间"
    )
    update_time: Mapped[datetime] = mapped_column(
        DateTime, server_default=func.now(), onupdate=func.now(), comment="更新时间"
    )


class CreatorMixin:
    create_user: Mapped[str] = mapped_column(
        String(50), default="admin", comment="创建用户"
    )
    update_user: Mapped[str] = mapped_column(
        String(50), onupdate="admin", comment="更新用户"
    )


class SoftDeleteMixin:
    is_delete: Mapped[bool] = mapped_column(default=False)



# Author: Jacky Lee
# Datetime: 2025-08-06 17:47:59
# Description: 将 format-json 转换为 inline-json，并复制到剪贴板


import json
import pprint

import inquirer
import pyperclip


def inline():
    pprint.pp("将格式化的 json 转为 单行json，并复制到剪贴板")
    KEY = "json-path"
    questions = [
        inquirer.Path(
            name=KEY,
            message="请给出需要一行的文件路径",
            path_type=inquirer.Path.FILE,
            exists=True,
        )
    ]
    answer = inquirer.prompt(questions)
    try:
        json_str_format = json.load(open(answer.get(KEY), "r"))
        json_str_inline = json.dumps(json_str_format)
        pyperclip.copy(json_str_inline)
        pprint.pp("成功复制到剪贴板")
    except Exception:
        pprint.pp("执行失败")


if __name__ == "__main__":
    inline()



from rich.table import Table


def build_table(title: str, header: list[str], rows: list):
    table = Table(show_header=True, header_style="bold magenta", title=title)
    for head in header:
        table.add_column(head)
    for row in rows:
        table.add_row(*row)
    return table



from __future__ import annotations

import queue
from typing import List


class TreeNode:
    def __init__(self, val=None, left=None, right=None):
        self.val = val
        self.left = left
        self.right = right


class Tree:
    def __init__(self, root: TreeNode = None) -> None:
        self.root = root

    def inorder(self, node: TreeNode, call=print):
        """
        traverse in-order
        """
        if node is None:
            return
        self.inorder(node.left) if node.left else 1
        call(node.val)
        self.inorder(node.right) if node.right else 1

    def preorder(self, node: TreeNode, call=print):
        """
        traverse pre-order
        """
        if node is None:
            return
        call(node.val)
        self.preorder(node.left) if node.left else 1
        self.preorder(node.right) if node.right else 1

    def postorder(self, node: TreeNode, call=print):
        """
        traverse post-order
        """
        if node is None:
            return
        self.postorder(node.left) if node.left else 1
        self.postorder(node.right) if node.right else 1
        call(node.val)

    def add_node(self, root, l_val, r_val):
        if root is None:
            return None, None
        l_node = None if l_val is None else TreeNode(l_val)
        r_node = None if r_val is None else TreeNode(r_val)
        root.left, root.right = l_node, r_node
        return l_node, r_node

    def add_tree(self, nums: List[int]):
        # nums = [3, 1, 5, 0, 2, 4, 6]
        i, n = 0, len(nums)
        q = queue.Queue()
        # build root node
        self.root = TreeNode(nums[0])
        self.val = nums[i]
        q.put(self.root)
        # build child root
        while i < n:
            node = q.get()
            l_val = None if 2 * i + 1 >= n else nums[2 * i + 1]
            r_val = None if 2 * i + 2 >= n else nums[2 * i + 2]
            l_node, r_node = self.add_node(node, l_val, r_val)
            q.put(l_node)
            q.put(r_node)
            i += 1
        return



from collections.abc import MutableMapping
from functools import reduce


def deepmerge(*dicts) -> dict:
    """TODO: 需要解释为什么需要这个方法"""

    def _deepmerge(source: dict, destination: dict) -> dict:
        """Updates two dicts of dicts recursively (https://stackoverflow.com/a/24088493/8965861)."""
        for k, v in source.items():
            if k in destination:
                # this next check is the only difference!
                if all(isinstance(e, MutableMapping) for e in (v, destination[k])):
                    destination[k] = deepmerge(v, destination[k])
                # we could further check types and merge as appropriate here.
        d3 = source.copy()
        d3.update(destination)
        return d3

    return reduce(_deepmerge, tuple(dicts))


def merge_dict_from_dict(base_dict: dict, new_dict: dict):
    """合并两个字典"""
    if new_dict:
        tmp_dict = {}
        for k, v in new_dict.items():
            tmp_dict[k] = v
        _d = deepmerge(base_dict, tmp_dict)
        base_dict.update(_d)
    return base_dict



import logging
from pathlib import Path

import pyperclip
from rich.console import Console

from pydevman.file.common import assert_path_exist_and_is_file

log = logging.getLogger(__name__)
console = Console()


def from_clipboard_or_file(src: Path) -> str:
    if src is None:
        return pyperclip.paste()
    assert_path_exist_and_is_file(src)
    return src.read_text()


def to_clipboard_or_file(dst: Path, content: str, force: bool, quiet=False) -> bool:
    # 写入剪贴板
    console.quiet = quiet
    if dst is None:
        pyperclip.copy(content)
        console.print("已复制到剪贴板")
        return
    # dst 非空,路径
    if dst.exists() and force:
        dst.write_text(content)
        console.print(f"已写入文件 {dst.name}")
    else:
        console.print(f"目标文件 {dst.name} 已存在，使用 -f 强制输出")



"""
create_time: 2022-04
author: Jacky Lee
description: 数独解密
"""


class Sudoku:
    def __init__(self, units: list[list[int]]) -> None:
        """
        units(9,9)
        """
        self.flag = False
        self.units = units

    def _validation(self, row: int, col: int, num: int) -> bool:
        """
        validation the value.
        """
        if self.units[row][col]:
            return False
        block_row = row // 3 * 3
        block_col = col // 3 * 3
        for unit in self.units:
            for j in range(9):
                if self.units[row][j] == num or unit[col] == num:
                    return False
        for i in range(block_row, block_row + 3):
            for j in range(block_col, block_col + 3):
                if self.units[i][j] == num:
                    return False
        return True

    def draw(self):
        """
        draw sudoku.
        """
        for row in self.units:
            print(row)

    def dfs(self, row: int, col: int):
        """
        deep first search
        """
        if row == 9:
            self.flag = True
            return
        elif self.units[row][col] > 0:
            self.dfs(row + (col + 1) // 9, (col + 1) % 9)
        for i in range(1, 10):
            if not self._validation(row, col, i):
                continue
            self.units[row][col] = i
            self.dfs(row + (col + 1) // 9, (col + 1) % 9)
            if self.flag:
                return
            self.units[row][col] = 0


def test_sudoku():
    """
    test sample
    """
    units = [
        [0, 0, 0, 8, 0, 0, 0, 0, 9],
        [0, 1, 9, 0, 0, 5, 8, 3, 0],
        [0, 4, 3, 0, 1, 0, 0, 0, 7],
        [4, 0, 0, 1, 5, 0, 0, 0, 3],
        [0, 0, 2, 7, 0, 4, 0, 1, 0],
        [0, 8, 0, 0, 9, 0, 6, 0, 0],
        [0, 7, 0, 0, 0, 6, 3, 0, 0],
        [0, 3, 0, 0, 7, 0, 0, 8, 0],
        [9, 0, 4, 5, 0, 0, 0, 0, 1],
    ]
    sudoku = Sudoku(units)
    sudoku.dfs(0, 0)
    sudoku.draw()


if __name__ == "__main__":
    test_sudoku()



from os import PathLike
from typing import Dict, Union

from omegaconf import OmegaConf

from pydevman.file.load import load_config_file


# 对于配置，建议开启环境，关闭 merge
def create_config(arg: Union[Dict, PathLike]) -> OmegaConf:
    """单文件配置导入，可以导入 json,toml,yaml 等文件"""
    _arg = arg
    if isinstance(arg, PathLike):
        _arg = load_config_file(arg)
    return OmegaConf.create(_arg)


def merge_config(config: OmegaConf, arg) -> None:
    """从路径中合并配置"""
    _config = create_config(arg)
    config.merge_with(_config)



import re
from datetime import datetime
from typing import Union

from pydevman.json.core import DelHtmlTagHandler, JsonProcessor, RecursiveHandler


def api_parse_str_to_json(
    text: str, *, recursive: bool, del_html_tag: bool
) -> Union[dict, list, str, int, float]:
    # 递归解析
    processor = JsonProcessor()
    if recursive:
        processor.register(RecursiveHandler())
    if del_html_tag:
        processor.register(DelHtmlTagHandler())

    parsed = processor.process(text)
    return processor.dump_readable(parsed)


def api_format_json_inline(text: str):
    processor = JsonProcessor()
    parsed = processor.process(text)
    return processor.dump_inline(parsed)


def api_dump_json_to_str(text: str):
    processor = JsonProcessor()
    return processor.dump_inline(text)


def get_possible_datetime_from_str(line: str) -> datetime:
    raise NotImplementedError("Not Finish.")
    dt_pat = re.compile(r"(\d{4}-?\d{2}-?\d{2}\s?\d{2}:?\d{2}:?\d{2}(\.\d{3})?)")
    return dt_pat


def get_possible_json_from_str(line: str) -> dict:
    raise NotImplementedError("Not Finish.")
    json_pat = re.compile(r"(?<!#)(\[?{.*}\]?)(?!#)")
    return json_pat


def parse_lines(lines: list[str]) -> list:
    raise NotImplementedError("Not Finish.")
    res = []
    for idx, line in enumerate(lines):
        get_possible_datetime_from_str(line)
        get_possible_json_from_str(line)
    return res



from pathlib import Path


def assert_path_not_exist(path: Path):
    assert path is not None, "参数不能为 None"
    assert isinstance(path, Path), "入参必须是路径"
    # src 必须不存在
    assert not path.exists(), f"${path} 必须不存在，但是该路径存在"


def assert_path_exist(path: Path):
    assert path is not None, "参数不能为 None"
    # src 必须存在
    assert path.exists(), f"${path} 必须存在，但是该路径不存在"


def assert_path_exist_and_is_file(file_path: Path):
    assert_path_exist(file_path)
    # src 必须是文件
    assert file_path.is_file(), f"${file_path} 文件必须是文件"


def assert_path_not_exist_and_is_file(file_path: Path):
    assert_path_not_exist(file_path)
    # src 必须是文件
    assert file_path.is_file(), f"${file_path} 文件必须是文件"


def assert_path_exist_and_is_dir(dir_path: Path):
    assert_path_exist(dir_path)
    # src 必须是文件夹
    assert dir_path.is_dir(), f"${dir_path} 文件必须是文件夹"


def assert_path_not_exist_and_is_dir(dir_path: Path):
    assert_path_not_exist(dir_path)
    # src 必须是文件夹
    assert dir_path.is_dir(), f"${dir_path} 文件必须是文件夹"


def is_empty_directory(dir: Path) -> bool:
    if not dir.is_dir():
        return False
    empty = True
    for item in dir.iterdir():
        # 递归查询子目录
        empty = empty and is_empty_directory(item)
    return empty


def is_dot_path(path: Path) -> bool:
    assert_path_exist(path)
    return path.name.startswith(".")



import configparser
import json
from pathlib import Path

import tomli
import yaml
from omegaconf import OmegaConf

from pydevman.file.common import assert_path_exist_and_is_file


def load_config_file(path):
    assert_path_exist_and_is_file(path)
    _path = Path(path)
    _suffix = _path.suffix
    if _suffix == ".json":
        return _parse_json(path)
    if _suffix in (".yaml", ".yml"):
        return _parse_yaml(path)
    if _suffix == ".toml":
        return _parse_toml(path)
    if _suffix == ".ini":
        return _parse_ini(path)
    raise TypeError("文件类型错误")


def _parse_json(_path):
    with open(_path, encoding="utf-8") as stream:
        return json.load(stream)


def _parse_yaml(_path):
    with open(_path, encoding="utf-8") as stream:
        return yaml.safe_load(stream)


def _parse_toml(_path):
    with open(_path, "rb") as stream:
        return tomli.load(stream)


def _parse_ini(_path):
    parser = configparser.ConfigParser()
    parser.read(_path, encoding="utf8")

    data = {section: dict(parser[section]) for section in parser.sections()}
    return OmegaConf.create(data)



"""
快速排序
"""

def _partition(A:list[int],p:int,q:int) -> int:
  """
  对数组原地排序
  参数：
    A (n,) : 长度为n的数组
    p int  : 左指针
    q int  : 右指针
  返回值：
    i      : 返回值是pivot的位置
  """
  x = A[p] # pivot
  i = p
  for j in range(p+1,q):
    if A[j] <= x:
      i += 1
      A[i], A[j] = A[j] , A[i]
  A[p], A[i] = A[i], A[p]
  return i

def _quick_sort(A:list[int],p:int,r:int) -> None:
  """
  快速排序，递归调用
  参数：
    A (n,) : 长度为n的数组
    p int  : 左指针
    r int  : 右指针
  返回值：
    None
  """
  if p < r:
    q = _partition(A,p,r)
    _quick_sort(A,p,q)
    _quick_sort(A,q+1,r)
  return A

def quick_sort(A):
  n = len(A)
  return _quick_sort(A,0,n)

def example_quick_sort():
  """
  案例展示
  """
  A = [6,10,13,5,8,3,2,11]
  n = len(A)
  _quick_sort(A,0,n)
  print(A)


"""
计数排序
"""

def counting_sort(a):
  k = max(a) - min(a)
  n = len(a)
  b = [0] * n
  c = [_ for _ in range(k+1)]
  for i in a:
    c[i] += 1
  for i in range(k):
    c[i] += c[i-1]
  for j in range(n,0,-1):
    b[c[a[j]]] = a[j]
    c[j] -= 1
  return b
def test_couting_sort():
  a = [4,1,3,4,3,0]
  print(counting_sort(a))
# test_couting_sort()

'''
基数排序
'''

def redix_sort(a):
  pass



import bisect


class SortedKeyDict:
    def __init__(self):
        self._dict = {}
        self._keys = []  # 升序排列

    def __setitem__(self, key, value):
        if key not in self._dict:
            bisect.insort_right(self._keys, key)
        self._dict[key] = value

    def __getitem__(self, key):
        return self._dict[key]

    def __delitem__(self, key):
        del self._dict[key]
        idx = bisect.bisect_left(self._keys, key)
        del self._keys[idx]

    def __iter__(self):
        # 遍历时降序
        for key in reversed(self._keys):
            yield key, self._dict[key]

    def items(self):
        for key in self:
            yield key, self._dict[key]

    def __len__(self):
        return len(self._keys)

    def get(self, key, default=None):
        return self._dict.get(key, default)



import csv
from itertools import islice
from typing import Generator, Iterable


def generate_csv_row(stream: Iterable, skip: int) -> Generator[dict, None, None]:
    """生成 csv 内容

    Args:
        stream (Iterable): 文件流或字符串列表
        skip (int): 跳过的行数

    Yields:
        dict: 生成字典内容
    """
    _iter = islice(stream, skip, None)
    _dict = csv.DictReader(_iter)
    for _line_dict in _dict:
        yield _line_dict



class Scaffold:
    def __init__(self, context):
        self.context = context

    def init_scaffold(self, config):
        self.config = config

    @property
    def debug(self) -> bool:
        return self.context.debug

    @property
    def env(self) -> str:
        return self.context.env



from pydantic import BaseModel

from pydevman.common.sorted_key_dict import SortedKeyDict
from pydevman.string.match import MatchStrategy, match_by_strategy


class Rule(BaseModel):
    name: str
    strategy: MatchStrategy
    patterns: list[str]
    priority: int = 0

    def match(self, text: str) -> bool:
        try:
            for pattern in self.patterns:
                if match_by_strategy(pattern, text, self.strategy):
                    return True
            return False
        except TypeError:
            return False
        except Exception as e:
            raise e


class RuleEngine:
    def __init__(self):
        self.rules = SortedKeyDict()

    def register(self, rule: Rule):
        priority = rule.priority
        li: list = self.rules.get(priority)
        if li is None:
            self.rules[priority] = [rule]
        else:
            self.rules[priority].append(rule)

    def match(self, text: str) -> list[Rule]:
        """返回匹配的规则列表"""
        for _, li in self.rules:
            li: list[Rule]
            matched = False
            matched_list = []
            for rule in li:
                if rule.match(text):
                    matched = True
                    matched_list.append(rule)
            if matched:
                return matched_list
        return matched_list



import typer

from pydevman.echo import app as echo_app
from pydevman.file.__main__ import app as file_app
from pydevman.json.__main__ import app as json_app

app = typer.Typer()


def main():
    app.add_typer(echo_app, name="echo", help="echo 工具")
    app.add_typer(json_app, name="json", help="json 工具")
    app.add_typer(file_app, name="file", help="file 工具")
    app()


if __name__ == "__main__":
    main()



from copy import deepcopy

from pydevman.helper.merge import deepmerge


def test_deepmerge_v2():
    # given:
    d1 = {"x": 1, "y": {"a": 1, "b": 2}}
    d2 = {"y": {"b": 99}}
    d3 = deepcopy(d1)
    d4 = deepcopy(d2)

    # when:
    r1 = deepmerge(d1, d2)
    d3.update(d4)

    # then:
    assert r1 != d3



"""
create_time: 2025-06-23 17:19:11
author: Jacky Lee
"""

import logging
import re
from pathlib import Path

import inquirer
import typer
from rich.console import Console

from pydevman.args import ARG_SRC
from pydevman.file.copy import copytree
from pydevman.file.delete import del_dir, del_empty_dir_recursive
from pydevman.file.move import move_match_pattern_file, move_prefix_ext
from pydevman.file.stat import SuffixStat, api_stat_cnt, api_stat_prefix

# from pydevman.query.query import query_check, query_list

console = Console()
app = typer.Typer()

log = logging.getLogger(__name__)


@app.command("stat-suffix", help="统计: 根据文件后缀统计文件")
def stat_suffix_query_interaction(src: ARG_SRC):
    # func = "stat-suffix"
    # src = query_list(func, "src", "请输入源文件夹目录")
    # suffix = query_check(func, "suffix", "请输入文件名后缀(非拓展名)")
    console.rule("根据 suffix 文件计数")
    # api_stat_suffix(Path(src), suffix)
    stat = SuffixStat(None)
    stat.stat_paths(src)
    table = stat.build_table()
    console.print(table)


@app.command("stat-cnt", help="统计: 统计文件夹中每个文件的数目")
def stat_cnt_query():
    func = "stat-cnt"
    src = query_list(func, "src", "请输入源文件夹目录")
    console.rule("根据目录统计递归文件")
    api_stat_cnt(Path(src))


@app.command("stat-prefix", help="统计: 根据 PREFIX 统计文件")
def stat_prefix_query():
    func = "stat-prefix"
    src = query_list(func, "src", "请输入源文件夹目录")
    ext = query_check(func, "ext", "请输入文件拓展名")
    console.rule("根据目录统计递归文件前缀")
    api_stat_prefix(Path(src), ext)


@app.command("copy-dir", help="删除: 删除 dst 文件夹内容并复制 src 内容")
def copy_dir_query():
    func = "copy-dir"
    console.rule("复制文件夹 SRC -> DST")
    src = query_list(func, "src", "请输入源文件夹目录")
    dst = query_list(func, "dst", "请输入目标文件夹目录")
    dry = inquirer.confirm("是否 DRY-RUN 模式", default=True)
    console.rule("高危操作，请谨慎操作⚠️")
    console.log(f"源文件夹='{src}' -> 目标文件夹='{dst}'")
    copytree(Path(src), Path(dst), dry)


@app.command("del-dir", help="删除: 删除 dst 文件夹内容到回收站")
def del_dir_query():
    console.rule("删除文件夹内容")
    func = "del-dir"
    dst = query_list(func, "dst", "请输入目标文件夹目录")
    dry = inquirer.confirm("是否 DRY-RUN 模式", default=True)
    del_dir(Path(dst), dry)


@app.command("del-empty-dir", help="删除: 删除目录中的空文件夹")
def del_empty_dir_query():
    func = "del-empty-dir"
    console.rule("删除空文件夹")
    dst = query_list(func, "dst", "请输入目标文件夹目录")
    dry = inquirer.confirm("是否 DRY-RUN 模式", default=True)
    del_empty_dir_recursive(Path(dst), dry)


@app.command("move-prefix-ext", help="删除: 按照文件末尾移动文件")
def move_prefix_ext_query():
    func = "move-prefix-ext"

    src = query_list(func, "src", "请输入源文件夹目录")
    dst = query_list(func, "dst", "请输入目标文件夹目录")
    prefix = query_list(func, "prefix", "请输入文件前缀")
    ext = query_check(func, "ext", "请输入文件拓展名")
    dry = inquirer.confirm("是否 DRY-RUN 模式", default=True)
    dry = inquirer.confirm("是否 DRY-RUN 模式", default=True)
    move_prefix_ext(
        Path(src), Path(dst), prefix=re.compile(prefix, re.I), ext=ext, dry=dry
    )


@app.command("move-pattern-dst", help="删除: 将文件移动到根目录")
def move_pattern_dst_query():
    func = "del-dir"
    src = query_list(func, "src", "请输入源文件夹目录")
    dst = Path(query_list(func, "dst", "请输入目标文件夹目录"))
    pattern = query_list(func, "pattern", "请输入正则表达式")
    dry = inquirer.confirm("是否 DRY-RUN 模式", default=True)
    move_match_pattern_file(Path(src), Path(dst), re.compile(pattern), dry)


if __name__ == "__main__":
    app()



import enum
from dataclasses import dataclass


class ResultCode(int, enum.Enum):
    SUCCESS = 0
    FAIL = 1
    ERROR = -1


@dataclass
class CommonResult:
    code: ResultCode = 0
    message: str = None
    data: object = None



"""
Author: Jacky Lee
Date: 2025-10-22
Description: 递归解析 json 代码
"""

import json
import logging
from abc import ABC, abstractmethod
from typing import Union

log = logging.getLogger(__name__)


class AbstractHandler(ABC):
    @abstractmethod
    def handle_dict(self, arg: dict): ...

    @abstractmethod
    def handle_list(self, arg: list): ...

    @abstractmethod
    def handle_str(self, arg: str): ...

    @abstractmethod
    def handle_int(self, arg: int): ...

    @abstractmethod
    def handle_float(self, arg: float): ...

    @abstractmethod
    def handle_bool(self, arg: bool): ...

    @abstractmethod
    def handle_none(self, arg): ...

    @abstractmethod
    def handle(self, arg): ...


class DefaultHandler(AbstractHandler):
    def handle_dict(self, arg):
        log.debug("handle dict...")
        return arg

    def handle_list(self, arg):
        log.debug("handle list...")
        return arg

    def handle_str(self, arg):
        log.debug("handle str...")
        return arg

    def handle_int(self, arg):
        log.debug("handle int...")
        return arg

    def handle_float(self, arg):
        log.debug("handle float...")
        return arg

    def handle_none(self, arg):
        log.debug("handle none...")
        return arg

    def handle(self, arg):
        log.debug("handle default ...")
        if arg is None:
            return self.handle_none(arg)
        if isinstance(arg, list):
            return self.handle_list(arg)
        elif isinstance(arg, dict):
            return self.handle_dict(arg)
        elif isinstance(arg, str):
            return self.handle_str(arg)
        # isinstance(False, int) >> true
        # isinstance(True, int) >> true
        # issubclass(bool, int) >> true
        elif isinstance(arg, int):
            return self.handle_int(arg)
        elif isinstance(arg, float):
            return self.handle_float(arg)
        else:
            raise TypeError(
                "Argument is not any of List, Object, String, Number or None."
            )


class RecursiveHandler(DefaultHandler):
    """递归解析,参数为 dict, list, str, int, float, None 中的一种"""

    def handle_dict(self, arg):
        _di = {}
        for k, v in arg.items():
            _di[k] = self.handle(v)
        return _di

    def handle_list(self, arg):
        _list = []
        for _, item in enumerate(arg):
            tmp = self.handle(item)
            _list.append(tmp)
        return _list

    def handle_str(self, arg) -> Union[dict, list, str, int, float, None]:
        try:
            # 如果可以被解析
            shallow_parsed = json.loads(arg)
        except json.JSONDecodeError:
            return arg
        deep_parsed = self.handle(shallow_parsed)
        return deep_parsed


class DelHtmlTagHandler(DefaultHandler):
    def handle_str(self, arg):
        import bs4

        return bs4.BeautifulSoup(arg, "html.parser").get_text()


class JsonProcessor:
    def __init__(self):
        self.handlers: list[AbstractHandler] = []

    def register(self, handler: AbstractHandler):
        self.handlers.append(handler)

    def process(self, text: str):
        try:
            _text = json.loads(text)
        except json.JSONDecodeError as e:
            log.error(f'无法解析字符串, text="{text}"')
            raise e

        for handler in self.handlers:
            _text = handler.handle(_text)
        return _text

    def dump_readable(self, obj: Union[str, dict, list]):
        return json.dumps(obj, indent=2, ensure_ascii=False)

    def dump_inline(self, obj: Union[str, dict, list]):
        return json.dumps(obj, ensure_ascii=False)



import json
import logging

import typer
from rich.console import Console
from typing_extensions import Annotated

from pydevman.args import ARG_DST, ARG_FORCE_COVER_DST, ARG_QUIET, ARG_SRC, ARG_VERBOSE
from pydevman.helper.interactive import from_clipboard_or_file, to_clipboard_or_file
from pydevman.json.api import (
    api_dump_json_to_str,
    api_format_json_inline,
    api_parse_str_to_json,
)
from pydevman.log import config_log

app = typer.Typer()
console = Console()

ARG_RECURSIVE = Annotated[
    bool,
    typer.Option("--recursive", "-r", help="是否递归去转义", show_default="默认递归"),
]

ARG_DEL_HTML_TAG = Annotated[
    bool,
    typer.Option("--del-tag", help="是否递归去转义", show_default="默认递归"),
]


@app.command("parse", help="解析字符串为 json(默认递归去转义)")
def recursive_parse_json(
    src: ARG_SRC = None,
    dst: ARG_DST = None,
    recursive: ARG_RECURSIVE = False,
    del_tag: ARG_DEL_HTML_TAG = False,
    force: ARG_FORCE_COVER_DST = False,
    verbose: ARG_VERBOSE = False,
    quiet: ARG_QUIET = False,
):
    # TODO: 解决模板代码的问题
    # TODO: 解决日志配置的问题
    console.quiet = quiet
    if verbose:
        config_log(logging.DEBUG)
    dump_content = None
    try:
        origin_content = from_clipboard_or_file(src)
        dump_content = api_parse_str_to_json(
            origin_content, recursive=recursive, del_html_tag=del_tag
        )
        to_clipboard_or_file(dst, dump_content, force, quiet)
        console.print(dump_content)
    except AssertionError as e:
        console.print("断言错误", e)
    except json.JSONDecodeError as e:
        console.print("无法解析字符串为 json", e)
    except Exception as e:
        console.print("未知异常", e)
        console.print("使用 -v 详细输出")


@app.command("inline", help="将 json 变为单行")
def format_json_inline(
    src: ARG_SRC = None,
    dst: ARG_DST = None,
    force: ARG_FORCE_COVER_DST = False,
    verbose: ARG_VERBOSE = False,
    quiet: ARG_QUIET = False,
):
    console.quiet = quiet
    if verbose:
        config_log(logging.DEBUG)
    dump_content = None
    try:
        origin_content = from_clipboard_or_file(src)
        dump_content = api_format_json_inline(origin_content)
        to_clipboard_or_file(dst, dump_content, force, quiet)
        # FIXME 解决 console 无法打印列表的问题
        console.print(dump_content)
    except AssertionError as e:
        console.print("断言错误", e)
    except json.JSONDecodeError as e:
        console.print("无法解析字符串为 json", e)
    except Exception as e:
        console.print("未知异常", e)
        console.print("使用 -v 详细输出")


@app.command("dump", help="将 json 序列化")
def dump_json_to_str(
    src: ARG_SRC = None,
    dst: ARG_DST = None,
    force: ARG_FORCE_COVER_DST = False,
    verbose: ARG_VERBOSE = False,
    quiet: ARG_QUIET = False,
):
    console.quiet = quiet
    if verbose:
        config_log(logging.DEBUG)
    dump_content = None
    try:
        origin_content = from_clipboard_or_file(src)
        dump_content = api_dump_json_to_str(origin_content)
        to_clipboard_or_file(dst, dump_content, force, quiet)
        console.print(dump_content)
    except AssertionError as e:
        console.print("断言错误", e)
    except json.JSONDecodeError as e:
        console.print("无法解析字符串为 json", e)
    except Exception as e:
        console.print("未知异常", e)
        console.print("使用 -v 详细输出")


if __name__ == "__main__":
    app()



from datetime import datetime
import enum


class DT_FORMAT(enum.Enum):
    DEFAULT_READABLE_PATTERN = "%Y-%m-%d %H:%M:%S"
    DEFAULT_SHORT_PATTERN = "%Y%m%d_%H%M%S"


class DT_FORMAT_LIST(enum.Enum):
    Y_YM_WITH_DASH = ["%Y", "%Y-%m"]
    Y_YM_YMD_WITH_DASH = ["%Y", "%Y-%m", "%Y-%m-%d"]
    Y_YM_YMDA_WITH_DASH = ["%Y", "%Y-%m", "%Y-%m-%d-%a"]


def timestamp_to_datetime_second(timestamp):
    return datetime.fromtimestamp(timestamp)


def timestamp_to_datetime_millisecond(timestamp):
    return datetime.fromtimestamp(timestamp / 1000)


def datetime_to_str(dt: datetime, format: str = DT_FORMAT.DEFAULT_READABLE_PATTERN):
    return dt.strftime(format)


def str_to_datetime(s: str, format: str = DT_FORMAT.DEFAULT_READABLE_PATTERN):
    return datetime.strptime(s, format)



from typing import Generic, List, Optional, Type, TypeVar, Union

from sqlalchemy import delete, select, update
from sqlalchemy.orm import DeclarativeBase, Session
from sqlalchemy.sql import ColumnElement

ModelType = TypeVar("ModelType", bound=DeclarativeBase)


class BaseMapper(Generic[ModelType]):
    def __init__(self, model: Type[ModelType]):
        assert issubclass(model, DeclarativeBase), "必须是 s.o.DeclarativeBase 的子类"
        assert hasattr(model, "id"), "必须有id"
        assert hasattr(model, "is_delete"), "必须有软删除字段"
        self.model: Type[ModelType] = model

    def select_by_id(self, session: Session, id: int) -> Union[ModelType, None]:
        """获取单个对象 by id"""
        return session.get(self.model, id)

    def select_by_condition(
        self, session: Session, condition: ColumnElement[bool]
    ) -> Union[ModelType, None]:
        """获取单个对象 by 条件(自动过滤软删除)"""
        stmt = select(self.model).where(self._not_soft_del).where(condition).limit(1)
        return session.scalars(stmt).first()

    def select_list(
        self,
        session: Session,
        condition: Optional[ColumnElement[bool]] = None,
        limit: Optional[int] = None,
    ) -> List[ModelType]:
        """批量获取"""
        stmt = select(self.model).where(self._not_soft_del)
        if condition:
            stmt = stmt.where(condition)
        if limit:
            stmt = stmt.limit(limit)
        return session.scalars(stmt).all()

    def insert(self, session: Session, po: ModelType) -> ModelType:
        """插入"""
        session.add(po)
        return po

    def insert_list(
        self, session: Session, po_list: List[ModelType]
    ) -> List[ModelType]:
        """批量插入"""
        for po in po_list:
            session.add(po)
        return po_list

    def update_by_po(self, session: Session, po: ModelType) -> ModelType:
        return session.merge(po)

    def update_by_id(self, session: Session, id: int, values: dict):
        stmt = update(self.model).where(self.model.id == id).values(**values)
        res = session.execute(stmt)
        return res.rowcount or 0

    def update_by_condition(
        self, session: Session, values: dict, condition: ColumnElement[bool]
    ) -> int:
        stmt = update(self.model).values(**values)
        if condition is not None:
            stmt = stmt.where(condition)
        res = session.execute(stmt)
        return res.rowcount or 0

    def upsert_by(self, session: Session, unique_field: str, po: ModelType):
        unique_value = getattr(po, unique_field)
        stmt = select(self.model).where(
            getattr(self.model, unique_field) == unique_value
        )
        existed = session.scalars(stmt).first()

        if existed:
            po.id = existed.id
            self.update_by_po(session, po)
            return po

        session.add(po)
        return po

    def delete_soft_by_condition(
        self, session: Session, condition: ColumnElement[bool]
    ) -> int:
        """软删除 by id"""
        return self.update_by_condition(session, {"is_delete": True}, condition)

    def delete_soft(self, session: Session, id: int) -> int:
        """软删除 by id"""
        return self.update_by_id(session, id, {"is_delete": True})

    def delete_by_condition(
        self, session: Session, condition: ColumnElement[bool]
    ) -> int:
        """硬删除 by id"""
        stmt = delete(self.model).where(condition)
        res = session.execute(stmt)
        return res.rowcount or 0

    def delete(self, session: Session, id: int) -> int:
        """硬删除 by id"""
        stmt = delete(self.model).where(self.model.id == id)
        res = session.execute(stmt)
        return res.rowcount or 0

    @property
    def _not_soft_del(self) -> ColumnElement[bool]:
        return self.model.is_delete.is_(False)



import logging

from rich.logging import RichHandler


def config_log(level: int = logging.INFO):
    # TODO: 使用 loguru
    logging.basicConfig(level=level, handlers=[RichHandler(rich_tracebacks=True)])



import enum
import re


class MatchStrategy(str, enum.Enum):
    EQUAL = "equal"
    INCLUDE = "include"
    PREFIX = "prefix"
    SUFFIX = "suffix"
    REGEX = "regex"
    ALWAYS = "always"


STR_LAST_NUM_PATTERN = re.compile(r"(\d+)(?=\D*$)")
DEFAULT_NUM_FILL_FORMAT = "{}-({})"


def match_by_strategy(pattern: str, string: str, strategy: str) -> bool:
    if strategy == MatchStrategy.EQUAL:
        return pattern.lower() == string.lower()
    if strategy == MatchStrategy.INCLUDE:
        return pattern.lower() in string.lower()
    if strategy == MatchStrategy.PREFIX:
        return string.lower().startswith(pattern.lower())
    if strategy == MatchStrategy.SUFFIX:
        return string.lower().endswith(pattern.lower())
    if strategy == MatchStrategy.REGEX:
        return re.match(pattern, string) is not None
    if strategy == MatchStrategy.ALWAYS:
        return True
    raise TypeError("未定义错误")


def match_str_last_num(string: str):
    """匹配最后一个字符串"""
    _match = re.search(STR_LAST_NUM_PATTERN, string)
    if _match:
        _match.group()
    _num = _match.group()
    if _num is None:
        return
    return _num


def increment_str_last_num(
    s: str, format: str = DEFAULT_NUM_FILL_FORMAT, default_num: int = 1
) -> str:
    num_s = match_str_last_num(s)
    if num_s is None:
        return format.format(s, default_num)
    # 存在前导零
    num = int(num_s)
    increment_str = str(num + 1).zfill(len(num_s))
    # FIXME: sub 类似 match ，但是这里是 search
    s2 = STR_LAST_NUM_PATTERN.sub(increment_str, s)
    return s2
